#!/usr/bin/env python

# Copyright 2017 Earth Sciences Department, BSC-CNS

# This file is part of Autosubmit.

# Autosubmit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Autosubmit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Autosubmit.  If not, see <http://www.gnu.org/licenses/>.
import os
import sys
import inspect
from flask_jsonpify import jsonify
import json
import logging
from sqlalchemy import create_engine
from flask_cors import CORS
from flask_restful import Resource, Api
from flask import Flask, request
from bscearth.utils.config_parser import ConfigParserFactory
from bscearth.utils.log import Log

sys.path.insert(0, os.path.abspath('.'))
# noinspection PyPackageRequirements
from database.db_common import search_experiment_by_id
from experiment.common_requests import get_experiment_data, get_experiment_graph, get_experiment_run
from experiment.common_requests import get_experiment_pkl, get_experiment_stats, test_run
from experiment.common_requests import get_experiment_tree_structured



#from flask.ext.jsonpify import jsonify


##sys.path.insert(0, os.path.abspath('.'))

app = Flask(__name__)
CORS(app)
api = Api(app)
gunicorn_logger = logging.getLogger('gunicorn.error')
app.logger.handlers = gunicorn_logger.handlers
app.logger.setLevel(gunicorn_logger.level)

@app.route('/expinfo/<string:expid>', methods=['GET'])
def exp_info(expid):
    app.logger.info('Received Query ' + expid)
    #print()
    result = get_experiment_data(expid)
    return result

@app.route('/search/<string:expid>', methods=['GET'])
def search_expid(expid):
    app.logger.info('Received Search ' + expid)
    result = search_experiment_by_id(expid)
    return result

# @app.route('/monitor/<string:expid>', methods=['GET'])
# def get_list(expid):
#     print("Received " + str(expid))
#     return ExpUtils.get_experiment_list(expid)

@app.route('/ifrun/<string:expid>', methods=['GET'])
def get_if_running(expid):
    app.logger.info('Received Running test ' + expid)
    result = test_run(expid)
    return result

@app.route('/graph/<string:expid>', methods=['GET'])
def get_list_format(expid):
    app.logger.info('Received Graph Query ' + expid)
    result = get_experiment_graph(expid)            
    return result

@app.route('/tree/<string:expid>', methods=['GET'])
def get_exp_tree(expid):
    app.logger.info('Received Tree Query ' + expid)
    # result = get_experiment_tree(expid)  
    result = get_experiment_tree_structured(expid)          
    return result

@app.route('/exprun/<string:expid>', methods=['GET'])
def get_experiment_running(expid):
    app.logger.info('Received Query Run ' + expid)
    result = get_experiment_run(expid)
    return result

@app.route('/pklinfo/<string:expid>/<string:timeStamp>', methods=['GET'])
def get_experiment_pklinfo(expid,timeStamp):
    app.logger.info('Received Query Pkl ' + expid + ' timeStamp ' + timeStamp)
    result = get_experiment_pkl(expid, timeStamp)
    return result

@app.route('/stats/<string:expid>/<string:filter_period>/<string:filter_type>')
def get_experiment_statistics(expid, filter_period, filter_type):
    app.logger.info("Received Stats Request " + expid + " hours: " + filter_period + " type: " + filter_type)
    result = get_experiment_stats(expid, filter_period, filter_type)
    return result

# @app.route('/group/<string:expid>/<string:groupby>')
# def get_experiment_grouped(expid, groupby):
#     app.logger.info("Received Group Request " + expid + " group by " + groupby)
#     result = get_experiment_graph_grouped(expid, groupby)
#     return result



# api.add_resource(Monitor, '/monitor') # Route Monitor


#if __name__ == '__main__':
    
#     app.run(host='0.0.0.0', port='8888')
